--============================================================================= 
--
-- Mode Controller
--
-- Toggles through choosing 1 out of 6 possible LED sequence modes
--
--============================================================================= 

library ieee;
	use ieee.std_logic_1164.all;
	use ieee.std_logic_unsigned.all;

entity control_mux is

	port (		
		clk     		: in 	std_logic;
		resetn   	: in 	std_logic;
		user_btn 	: in	std_logic;
		mode   		: out std_logic_vector(2 downto 0);
		led_in		: in	std_logic_vector(15 downto 0) := (others => '0');
		led_out		: out std_logic_vector(7 downto 0)	
	);
	
end entity;

architecture behavioural of control_mux is

	signal cnt 		: 		std_logic_vector(2 downto 0) := "001";
	signal reset	:		std_logic :='1';
	signal keyout	: 		std_logic;
	signal keydeb 	: 		std_logic := '0';
	signal debcnt 	: 		integer range 0 to 63 := 0;
	
begin

	-- debounce key
   process 
	begin
      wait until rising_edge(clk);
      if (USER_BTN=keydeb) then 
			debcnt <= 0;
      else                   
			debcnt <= debcnt+1;
      end if;
      if (debcnt=63) then 
			keydeb <= USER_BTN; 
      end if;
   end process;
   keyout <= keydeb;
	

	-- toggle through states
	process(clk,keyout, resetn)
	begin
		if resetn = '0' or reset ='0' then
			cnt <= "001";
				
		elsif falling_edge(keyout) then
				cnt <= cnt + 1;	
			if cnt >= "101" then
				cnt <= "001";
			end if;
		end if;
	end process;

	

	process(clk,cnt)												-- Pass LED sequence to LEDs depending on mode
	begin	
		if led_in (15 downto 8) = "01010101" then
			reset <= '0';
			led_out <= led_in (15 downto 8);
		else
			reset <= '1';
			case cnt is	
				when "001" => led_out <= led_in (15 downto 8);
				when "010" => led_out <= led_in (7 downto 0);
				when "011" => led_out <= led_in (15 downto 8);
				when "100" => led_out <= led_in (15 downto 8);
				when "101" => led_out <= led_in (15 downto 8);
				when others => led_out <="00000000";
			end case;
		end if;
	end process;
	
	mode <= cnt;

end architecture;